@@ -701,3 +701,14 @@ all_names <- function(expr) {
 is_constant <- function(expr) {
   length(all_cols(expr)) == 0
 }
+
+remove_enclosing_parentheses <- function(expr) {
+  # note: this might fail to remove enclosing parentheses
+  #   if expr contains parentheses inside quoted strings
+  # for example: queryparser:::remove_enclosing_parentheses("(') (')")
+  while (grepl("^\\(.*\\)$", expr)) {
+    if (length(gregexpr("\\((?>[^()]|(?R))*\\)", expr, perl = TRUE)[[1]]) > 1) break
+    expr <- sub("^\\((.*)\\)$", "\\1", expr)
+  }
+  expr
+}
