@@ -51,8 +51,7 @@ test_that("parse_query(tidy = FALSE) works on 'flights' example query", {
       where = list(quote((distance >= 200 & distance <= 300) &
       !is.na(air_time))), group_by = list(quote(origin), quote(dest)),
       having = list(quote(num_flts > 3000)), order_by = structure(list(
-      quote(num_flts), quote(avg_delay)), decreasing = c(TRUE,
-      TRUE), aggregate = c(FALSE, FALSE)), limit = list(100L)), aggregate = TRUE)
+      quote(-xtfrm(num_flts)), quote(-xtfrm(avg_delay))), aggregate = c(FALSE, FALSE)), limit = list(100L)), aggregate = TRUE)
   )
 })
 
@@ -320,6 +319,48 @@ test_that("parse_query() works on SELECT ALL example query", {
   )
 })
 
+test_that("parse_query() works with NULLS FIRST in ORDER BY", {
+  expect_equal(
+    parse_query("SELECT x FROM df ORDER BY x NULLS FIRST"),
+    list(select = list(quote(x)),
+      from = list(quote(df)),
+      order_by = list(quote(!is.na(x)), quote(x)))
+  )
+})
+
+test_that("parse_query() works with NULLS LAST in ORDER BY", {
+  expect_equal(
+    parse_query("SELECT x FROM df ORDER BY x NULLS LAST"),
+    list(select = list(quote(x)),
+      from = list(quote(df)),
+      order_by = list(quote(is.na(x)), quote(x)))
+  )
+})
+
+test_that("parse_query() works with ASC/DESC and NULLS FIRST/LAST in ORDER BY", {
+  expect_equal(
+    parse_query("SELECT w, x, y, z FROM df
+      ORDER BY w ASC NULLS FIRST, x DESC NULLS FIRST, y ASC NULLS LAST, z DESC NULLS LAST"),
+    list(select = list(quote(w), quote(x), quote(y), quote(z)), from = list(quote(df)),
+      order_by = list(quote(!is.na(w)), quote(w), quote(!is.na(x)),
+      quote(-xtfrm(x)), quote(is.na(y)), quote(y), quote(is.na(z)),
+      quote(-xtfrm(z))))
+  )
+})
+
+test_that("parse_query(tidy = TRUE) works with ASC/DESC and NULLS FIRST/LAST in ORDER BY", {
+  expect_equal(
+    parse_query("SELECT w, x, y, z FROM df
+        ORDER BY w ASC NULLS FIRST, x DESC NULLS FIRST, y ASC NULLS LAST, z DESC NULLS LAST",
+      tidyverse = TRUE
+    ),
+    list(select = list(quote(w), quote(x), quote(y), quote(z)), from = list(quote(df)),
+      order_by = list(quote(!is.na(w)), quote(w), quote(!is.na(x)),
+      str2lang("dplyr::desc(x)"), quote(is.na(y)), quote(y), quote(is.na(z)),
+      str2lang("dplyr::desc(z)")))
+  )
+})
+
 test_that("parse_query() stops on positional column references in ORDER BY clause", {
   expect_error(
     parse_query("SELECT x, y, z FROM t ORDER BY 1 DESC, 2"),
@@ -344,14 +385,28 @@ test_that("parse_query() stops when unvalid expression used in a SELECT DISTINCT
 test_that("parse_query() stops on incomplete CAST expression", {
   expect_error(
     parse_query("SELECT CAST(x) FROM y"),
-    "cast"
+    "CAST"
   )
 })
 
 test_that("parse_query() stops on malformed CAST expression", {
   expect_error(
     parse_query("SELECT CAST(x) AS y FROM z"),
-    "cast"
+    "CAST"
+  )
+})
+
+test_that("parse_query() stops on incomplete TRY_CAST expression", {
+  expect_error(
+    parse_query("SELECT TRY_CAST(x) FROM y"),
+    "TRY_CAST"
+  )
+})
+
+test_that("parse_query() stops on malformed TRY_CAST expression", {
+  expect_error(
+    parse_query("SELECT TRY_CAST(x) AS y FROM z"),
+    "TRY_CAST"
   )
 })
 
@@ -411,6 +466,20 @@ test_that("parse_query() stops when invalid use of DESC in ORDER BY clause", {
   )
 })
 
+test_that("parse_query() stops when invalid use of NULLS FIRST in ORDER BY clause", {
+  expect_error(
+    parse_query("SELECT x FROM y ORDER BY x, NULLS FIRST"),
+    "^Invalid"
+  )
+})
+
+test_that("parse_query() stops when invalid use of NULLS LAST in ORDER BY clause", {
+  expect_error(
+    parse_query("SELECT x FROM y ORDER BY x, NULLS LAST"),
+    "^Invalid"
+  )
+})
+
 test_that("parse_query() stops when expression in LIMIT clause", {
   expect_error(
     parse_query("SELECT x FROM y LIMIT sqrt(100)"),
@@ -519,8 +588,7 @@ test_that("parse_query() removes table name prefixes in single-table queries", {
       parse_query(query)
     },
     list(select = list(quote(year), quote(month), quote(day)), from = list(quote(flights)),
-      where = list(quote(arr_delay <= 0)), order_by = structure(list(quote(carrier)),
-      decreasing = FALSE))
+      where = list(quote(arr_delay <= 0)), order_by = structure(list(quote(carrier))))
   )
 })
 
@@ -534,8 +602,7 @@ test_that("parse_query() removes table alias prefixes in single-table queries",
       parse_query(query)
     },
     list(select = list(quote(year), quote(month), quote(day)), from = list(f = quote(flights)),
-      where = list(quote(arr_delay <= 0)), order_by = structure(list(quote(carrier)),
-      decreasing = FALSE))
+      where = list(quote(arr_delay <= 0)), order_by = structure(list(quote(carrier))))
   )
 })
 
@@ -552,3 +619,17 @@ test_that("parse_query() stops when table alias is disallowed", {
     "disallowed"
   )
 })
+
+test_that("parse_query() succeeds when parentheses around table name", {
+  expect_error(
+    parse_query("SELECT * FROM (ex)"),
+    NA
+  )
+})
+
+test_that("parse_query() succeeds when parentheses around table name and alias after", {
+  expect_error(
+    parse_query("SELECT * FROM (ex) x"),
+    NA
+  )
+})
